/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.dhcp.parser
 * File:    LeaseParser.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* Generated By:JavaCC: Do not edit this line. LeaseParser.java */
package de.fhhannover.inform.iron.dhcp.parser;

import java.io.*;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.text.SimpleDateFormat;

public class LeaseParser extends Parser implements LeaseParserConstants {
 private String path;
 private StringBuffer buf;
 private static int count;
 public static ArrayList<Lease> leases = new ArrayList<Lease>();
 public static ArrayList<Token> tokens = new ArrayList<Token>();

 /**
 * Sets the path of the file, which will be read
 *
 * @param an absolute path
 */
 public void setPath(String path) {
         this.path = path;
 }

 /**
 * Returns the absolute path of the file
 *
 * @return an absolute path
 */
 public String getPath() {
                return path;
 }

 /** 
 * Creates a BufferedReader for reading the file contents
 * 
 * @param an absolute path
 *
 */
 public static BufferedReader getFile(String path){
                BufferedReader file = null;
                try{
                        file = new BufferedReader(new FileReader(path));
                }
                 catch(Exception e) {
                        e.printStackTrace();
                }
                return file;
 }

 /**
 * Get all leases-objects found in the file
 *
 * @return An ArrayList with leases, null if no lease found
 */
 public ArrayList<Lease> getLeases() {
                return leases;
        }

 /**
 * Starts the parsing process. All leases found will be saved in an ArrayList 
 * 
 * @return An ArrayList with leases, null if no lease found
 */
 protected ArrayList<Lease> parseFile(){
                try {
                LeaseParser.leases.clear();
                count = 0;
                 return this.all();
                 }
                 catch(Exception e) {
                        e.printStackTrace();
                }
                return null;
        }

 /**
 * Creates a calendar-object from a given String
 *
 * @param
 * @return
 */
 private static Calendar getDateFromString(String date)
        {
                //2009/10/15 16:27:26
                String pattern = "F y/M/d H:m:s";
                SimpleDateFormat form = new SimpleDateFormat(pattern);
                Calendar cal = Calendar.getInstance();
                Date datum = null;
                try{
                        datum = form.parse(date);
                }
                catch(java.text.ParseException err){

                }
                cal.setTime(datum);
                return cal;
        }

/**
* Here come the productions for the parser
*/

//get all leases
  static final public ArrayList<Lease> all() throws ParseException {
    label_1:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case LEASE_CONST:
        ;
        break;
      default:
        jj_la1[0] = jj_gen;
        break label_1;
      }
      lease();
  count += 1;
    }
    jj_consume_token(0);
   {if (true) return leases;}
    throw new Error("Missing return statement in function");
  }

// get a lease
  static final public void lease() throws ParseException {
Token ip;
Option opt;
    jj_consume_token(LEASE_CONST);
    ip = jj_consume_token(IP);
        leases.add(new Lease(ip.image));
    jj_consume_token(BRACKET_LEFT);
    jj_consume_token(EOL);
    label_2:
    while (true) {
      opt = opts();
   if(opt != null)
                leases.get(count).addOption(opt);
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case OPTION_HARDWARE_ETHERNET:
      case OPTION_STARTS:
      case OPTION_ENDS:
      case OPTION_CLTT:
      case OPTION_BINDING_STATE:
      case OPTION_NEXT_BINDING_STATE:
      case STRING:
        ;
        break;
      default:
        jj_la1[1] = jj_gen;
        break label_2;
      }
    }
    jj_consume_token(BRACKET_RIGHT);
    label_3:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case EOL:
        ;
        break;
      default:
        jj_la1[2] = jj_gen;
        break label_3;
      }
      jj_consume_token(EOL);
    }
  }

// get the options
  static final public Option opts() throws ParseException {
 Token ident;
String val;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case OPTION_HARDWARE_ETHERNET:
      ident = jj_consume_token(OPTION_HARDWARE_ETHERNET);
      break;
    case OPTION_STARTS:
      ident = jj_consume_token(OPTION_STARTS);
      break;
    case OPTION_ENDS:
      ident = jj_consume_token(OPTION_ENDS);
      break;
    case OPTION_CLTT:
      ident = jj_consume_token(OPTION_CLTT);
      break;
    case OPTION_BINDING_STATE:
      ident = jj_consume_token(OPTION_BINDING_STATE);
      break;
    case OPTION_NEXT_BINDING_STATE:
      ident = jj_consume_token(OPTION_NEXT_BINDING_STATE);
      break;
    case STRING:
      ident = jj_consume_token(STRING);
      break;
    default:
      jj_la1[3] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
    val = value();
Option op = null;

//we handle some options special
if(ident.kind == OPTION_STARTS){
        leases.get(count).setStart(getDateFromString(val));
}
else if(ident.kind == OPTION_ENDS){
        leases.get(count).setEnd(getDateFromString(val));
}
else if(ident.kind == OPTION_HARDWARE_ETHERNET){
        leases.get(count).setMac(val);
}
else if(ident.kind == OPTION_BINDING_STATE){
        leases.get(count).setBindingState(val);
}
else{
op = new Option(ident.image);
op.addValue(val);
}
    label_4:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case EOL:
        ;
        break;
      default:
        jj_la1[4] = jj_gen;
        break label_4;
      }
      jj_consume_token(EOL);
    }
{if (true) return op;}
    throw new Error("Missing return statement in function");
  }

// get the value for an option
  static final public String value() throws ParseException {
 Token v = null;
StringBuffer buf = new StringBuffer();
    label_5:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case LEASE_TIME:
      case STRING:
      case QOUTED_STRING:
        ;
        break;
      default:
        jj_la1[5] = jj_gen;
        break label_5;
      }
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case STRING:
        v = jj_consume_token(STRING);
        break;
      case LEASE_TIME:
        v = jj_consume_token(LEASE_TIME);
        break;
      case QOUTED_STRING:
        v = jj_consume_token(QOUTED_STRING);
        break;
      default:
        jj_la1[6] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
    }
                        if(v != null)
                                buf.append(v.image);
    jj_consume_token(23);
{if (true) return buf.toString();}
    throw new Error("Missing return statement in function");
  }

  static private boolean jj_initialized_once = false;
  /** Generated Token Manager. */
  static public LeaseParserTokenManager token_source;
  static SimpleCharStream jj_input_stream;
  /** Current token. */
  static public Token token;
  /** Next token. */
  static public Token jj_nt;
  static private int jj_ntk;
  static private int jj_gen;
  static final private int[] jj_la1 = new int[7];
  static private int[] jj_la1_0;
  static {
      jj_la1_init_0();
   }
   private static void jj_la1_init_0() {
      jj_la1_0 = new int[] {0x800,0xfe000,0x40,0xfe000,0x40,0x180400,0x180400,};
   }

  /** Constructor with InputStream. */
  public LeaseParser(java.io.InputStream stream) {
     this(stream, null);
  }
  /** Constructor with InputStream and supplied encoding */
  public LeaseParser(java.io.InputStream stream, String encoding) {
    if (jj_initialized_once) {
      System.out.println("ERROR: Second call to constructor of static parser.  ");
      System.out.println("       You must either use ReInit() or set the JavaCC option STATIC to false");
      System.out.println("       during parser generation.");
      throw new Error();
    }
    jj_initialized_once = true;
    try { jj_input_stream = new SimpleCharStream(stream, encoding, 1, 1); } catch(java.io.UnsupportedEncodingException e) { throw new RuntimeException(e); }
    token_source = new LeaseParserTokenManager(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 7; i++) jj_la1[i] = -1;
  }

  /** Reinitialise. */
  static public void ReInit(java.io.InputStream stream) {
     ReInit(stream, null);
  }
  /** Reinitialise. */
  static public void ReInit(java.io.InputStream stream, String encoding) {
    try { jj_input_stream.ReInit(stream, encoding, 1, 1); } catch(java.io.UnsupportedEncodingException e) { throw new RuntimeException(e); }
    token_source.ReInit(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 7; i++) jj_la1[i] = -1;
  }

  /** Constructor. */
  public LeaseParser(java.io.Reader stream) {
    if (jj_initialized_once) {
      System.out.println("ERROR: Second call to constructor of static parser. ");
      System.out.println("       You must either use ReInit() or set the JavaCC option STATIC to false");
      System.out.println("       during parser generation.");
      throw new Error();
    }
    jj_initialized_once = true;
    jj_input_stream = new SimpleCharStream(stream, 1, 1);
    token_source = new LeaseParserTokenManager(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 7; i++) jj_la1[i] = -1;
  }

  /** Reinitialise. */
  static public void ReInit(java.io.Reader stream) {
    jj_input_stream.ReInit(stream, 1, 1);
    token_source.ReInit(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 7; i++) jj_la1[i] = -1;
  }

  /** Constructor with generated Token Manager. */
  public LeaseParser(LeaseParserTokenManager tm) {
    if (jj_initialized_once) {
      System.out.println("ERROR: Second call to constructor of static parser. ");
      System.out.println("       You must either use ReInit() or set the JavaCC option STATIC to false");
      System.out.println("       during parser generation.");
      throw new Error();
    }
    jj_initialized_once = true;
    token_source = tm;
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 7; i++) jj_la1[i] = -1;
  }

  /** Reinitialise. */
  public void ReInit(LeaseParserTokenManager tm) {
    token_source = tm;
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 7; i++) jj_la1[i] = -1;
  }

  static private Token jj_consume_token(int kind) throws ParseException {
    Token oldToken;
    if ((oldToken = token).next != null) token = token.next;
    else token = token.next = token_source.getNextToken();
    jj_ntk = -1;
    if (token.kind == kind) {
      jj_gen++;
      return token;
    }
    token = oldToken;
    jj_kind = kind;
    throw generateParseException();
  }


/** Get the next Token. */
  static final public Token getNextToken() {
    if (token.next != null) token = token.next;
    else token = token.next = token_source.getNextToken();
    jj_ntk = -1;
    jj_gen++;
    return token;
  }

/** Get the specific Token. */
  static final public Token getToken(int index) {
    Token t = token;
    for (int i = 0; i < index; i++) {
      if (t.next != null) t = t.next;
      else t = t.next = token_source.getNextToken();
    }
    return t;
  }

  static private int jj_ntk() {
    if ((jj_nt=token.next) == null)
      return (jj_ntk = (token.next=token_source.getNextToken()).kind);
    else
      return (jj_ntk = jj_nt.kind);
  }

  static private java.util.List<int[]> jj_expentries = new java.util.ArrayList<int[]>();
  static private int[] jj_expentry;
  static private int jj_kind = -1;

  /** Generate ParseException. */
  static public ParseException generateParseException() {
    jj_expentries.clear();
    boolean[] la1tokens = new boolean[24];
    if (jj_kind >= 0) {
      la1tokens[jj_kind] = true;
      jj_kind = -1;
    }
    for (int i = 0; i < 7; i++) {
      if (jj_la1[i] == jj_gen) {
        for (int j = 0; j < 32; j++) {
          if ((jj_la1_0[i] & (1<<j)) != 0) {
            la1tokens[j] = true;
          }
        }
      }
    }
    for (int i = 0; i < 24; i++) {
      if (la1tokens[i]) {
        jj_expentry = new int[1];
        jj_expentry[0] = i;
        jj_expentries.add(jj_expentry);
      }
    }
    int[][] exptokseq = new int[jj_expentries.size()][];
    for (int i = 0; i < jj_expentries.size(); i++) {
      exptokseq[i] = jj_expentries.get(i);
    }
    return new ParseException(token, exptokseq, tokenImage);
  }

  /** Enable tracing. */
  static final public void enable_tracing() {
  }

  /** Disable tracing. */
  static final public void disable_tracing() {
  }

}
